#include <stdio.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/kd.h>

static struct 	termios save_termios, buf ;
char 		consolemode;
int udelay = 5000; //5 milliseconds
char saveled;

char *prgname;

/* function prototypes */
void keyboard_init(void);			/* has to be called before any of the other functions */

int sendleds(unsigned char leds, int delay);
int sendbyte(unsigned char byte);
int restoreleds();

int tty_cbreak();		/* put terminal into a cbreak mode */
int tty_raw();			/* put terminal into a raw mode */
int tty_reset();		/* restore terminal mode */


/* implementations */
void keyboard_init(void)			/* has to be called before any of the other functions */
{
  // Saving current LEDs setting
  if ( ioctl(STDIN_FILENO, KDGETLED, &saveled) ){
    fprintf(stderr, "%s: ioctl() KDGETLED failed : %s\n", prgname, strerror(errno));
    fprintf(stderr, "You need to run this from the console, an xterm will not do!\n");
    exit(2); 
  };

  // Saving the current keyboard settings;
  if ( ioctl(STDIN_FILENO, KDGKBMODE , &consolemode) ){
    fprintf(stderr, "ioctl() KDGKBMODE failed : %s\n", strerror(errno));
    exit(2);
  }
  
  if (tcgetattr(STDIN_FILENO,&save_termios) < 0){
  	fprintf(stderr,"keyboard_init malfunction\n");
  }
  buf = save_termios; /* structure copy */
  
}

int sendleds(unsigned char leds, int delay)
{
  if (ioctl(STDIN_FILENO, KDSETLED,leds)){
    fprintf(stderr, "%s: ioctl() KDSETLED failed : %s\n", prgname, strerror(errno));
  }
  usleep(delay); // sleep for udely microseconds
}

int sendbyte(unsigned char byte)
{
  printf("sending byte: %i\n",byte);
  sendleds( byte & 7 ,udelay);
  sendleds( (byte>>3) & 7 ,udelay);
  sendleds( (byte>>6) & 3 ,udelay);
}

int restoreleds()
{
  if (sendleds(saveled,udelay)){
    fprintf(stderr, "Could not restore original led settings\n" );
    return 2;
  };
  return 0;
}


int tty_cbreak() /* put terminal into a cbreak mode */
{
  if ( ioctl(STDIN_FILENO, KDSKBMODE , K_XLATE) ){ 
    fprintf(stderr, "ioctl() KDSKBMODE failed : %s\n", strerror(errno));
    exit(2);
  };
  

  buf = save_termios; /* structure copy */

  buf.c_lflag &= ~( ECHO | ICANON); /* echo off, canonical mode off */
  buf.c_cc[VMIN] = 1; /* one byte at a time */
  buf.c_cc[VTIME] = 0; /* no timer */

  if (tcsetattr(STDIN_FILENO, TCSANOW, &buf) < 0)
  	return (-1);
  return (0);
}



int tty_raw() /* put terminal into a raw mode */
{
  if ( ioctl(STDIN_FILENO, KDSKBMODE , K_RAW) ){
    fprintf(stderr, "ioctl() KDSKBMODE failed : %s\n", strerror(errno));
    exit(2);
  };
  
  tcgetattr(STDIN_FILENO,&buf); 
  cfmakeraw(&buf);
  
  buf.c_cc[VMIN] = 1; /* one byte at a time */
  buf.c_cc[VTIME] = 0; /* no timer */

  if (tcsetattr(STDIN_FILENO, TCSANOW, &buf) < 0)
    fprintf(stderr, "itcsetattr(STDIN_FILENO, TCSANOW, &buf) failed : %s\n", strerror(errno));
  return (0);
}


int tty_reset() /* restore terminal mode */
{
  
  ioctl(STDIN_FILENO, KDSKBMODE , consolemode);
  if (tcsetattr(STDIN_FILENO, TCSANOW, &save_termios) < 0)
	return (-1);
  buf = save_termios; /* structure copy */
  return (0);
}
